package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.report.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 报表服务 service 实现
 *
 * @author xuwenjin
 * @Date 2019-04-22
 */
@Component
public class ReportServiceImpl implements ReportService {

    @Autowired
    private ReportSubjectService reportSubjectService;

    @Autowired
    private ReportSubjectRangeService reportSubjectRangeService;

    @Autowired
    private ReportSubjectCollegeService reportSubjectCollegeService;

    @Autowired
    private ReportSubjectTeacherService reportSubjectTeacherService;

    @Autowired
    private ReportSubjectTeacherClassService reportSubjectTeacherClassService;

    @Autowired
    private ReportSubjectClassService reportSubjectClassService;

    @Autowired
    private ReportSubjectQuestionService reportSubjectQuestionService;

    @Autowired
    private ReportSubjectGroupService reportSubjectGroupService;

    @Autowired
    private ExamSubjectService examSubjectService;


    /**
     * 生成报表数据
     *
     * @param examId 考试id
     */
    @Override
    public void generateReportData(Integer examId) {

        deleteData(examId);

        saveData(examId);

    }

    /**
     * 删除数据
     *
     * @param examId
     */
    private void deleteData(Integer examId) {

        reportSubjectService.deleteByExamId(examId);

        reportSubjectRangeService.deleteByExamId(examId);

        reportSubjectCollegeService.deleteByExamId(examId);

        reportSubjectTeacherService.deleteByExamId(examId);

        reportSubjectTeacherClassService.deleteByExamId(examId);

        reportSubjectClassService.deleteByExamId(examId);

        reportSubjectQuestionService.deleteByExamId(examId);

        reportSubjectGroupService.deleteByExamId(examId);

    }

    /**
     * 重新生成数据
     *
     * @param examId
     */
    private void saveData(Integer examId) {
        List<ExamSubject> examSubjects = examSubjectService.list(examId);

        //保存课程总量统计分析
        reportSubjectService.saveReportSubjectData(examId);

        //保存课程分段统计
        reportSubjectRangeService.saveReportRangeSubjectData(examId);

        for (ExamSubject subject : examSubjects) {
            String subjectCode = subject.getCode();
            //保存学院成绩分析
            reportSubjectCollegeService.saveReportSubjectCollegeData(examId, subjectCode);

            //保存课程任课老师成绩分析
            reportSubjectTeacherService.saveReportSubjectTeacherData(examId, subjectCode);

            //保存任课老师班级成绩分析
            reportSubjectTeacherClassService.saveReportSubjectTeacherClassData(examId, subjectCode);

            //保存成班级成绩分析
            reportSubjectClassService.saveReportSubjectClassData(examId, subjectCode);

            //保存客观题成绩分析
            reportSubjectQuestionService.saveReportSubjectObjectiveQuestionData(examId, subjectCode, true);

            //保存主观题成绩分析
            reportSubjectQuestionService.saveReportSubjectObjectiveQuestionData(examId, subjectCode, false);

            //保存大题型成绩分析
            reportSubjectGroupService.saveReportSubjectGroupData(examId, subjectCode);
        }
    }
}
