package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.ExamSubjectDao;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectClassDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectClass;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectClassService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.LinkedList;
import java.util.List;

/**
 * 课程班级分析 service实现
 *
 * @author xuwenjin
 * @date 2019-04-16
 */
@Service
public class ReportSubjectClassServiceImpl extends BaseQueryService<ReportSubjectClass> implements ReportSubjectClassService {

    @Autowired
    private ReportSubjectClassDao reportSubjectClassDao;

    @Autowired
    private ExamSubjectDao examSubjectDao;


    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectClassDao.deleteByExamId(examId);
    }

    @Override
    public List<ReportSubjectClass> findByQuery(final ReportSubjectQuery query) {
        List<ReportSubjectClass> result = reportSubjectClassDao.findAll(new Specification<ReportSubjectClass>() {
            @Override
            public Predicate toPredicate(Root<ReportSubjectClass> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();

                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (query.getSubjectCode() != null) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (StringUtils.isNotBlank(query.getClassName())) {
                    predicates.add(cb.equal(root.get("className"), query.getClassName()));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        });
        return result;
    }

    @Override
    public void saveReportSubjectClassData(Integer examId, String subjectCode) {
        ExamSubject examSubject = examSubjectDao.findByExamIdAndCode(examId, subjectCode);
        List<Object[]> reportSubjecCollegetList = reportSubjectClassDao.queryReportSubjectClassData(examId, subjectCode);
        for (Object[] rs : reportSubjecCollegetList) {
            ReportSubjectClass subject = new ReportSubjectClass();
            subject.setExamId(examId);
            subject.setSubjectCode(subjectCode);
            subject.setSubjectName(examSubject.getName());
            subject.setClassName(rs[0].toString());
            subject.setAvgScore(DoubleUtil.format(Double.parseDouble(rs[1].toString()), "0.00"));
            subject.setMaxScore(Double.parseDouble(rs[2].toString()));
            subject.setMinScore(Double.parseDouble(rs[3].toString()));
            subject.setPassCount(Integer.parseInt(rs[4].toString()));
            subject.setPassRate(DoubleUtil.div(subject.getPassCount(), Integer.parseInt(rs[6].toString()), 2));
            subject.setExcellentCount(Integer.parseInt(rs[5].toString()));
            subject.setExcellentRate(DoubleUtil.div(subject.getExcellentCount(), Integer.parseInt(rs[6].toString()), 2));
            reportSubjectClassDao.save(subject);
        }
    }

}
