package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.ExamQuestionDao;
import cn.com.qmth.stmms.biz.exam.dao.ExamStudentDao;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectQuestionDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.LinkedList;
import java.util.List;

/**
 * 客、主观题分析 service 实现
 *
 * @author xuwenjin
 * @Date 2019-04-17
 */
@Service
public class ReportSubjectQuestionServiceImpl extends BaseQueryService<ReportSubjectQuestion> implements ReportSubjectQuestionService {

    @Autowired
    private ReportSubjectQuestionDao reportSubjectQuestionDao;

    @Autowired
    private ExamStudentDao examStudentDao;

    @Autowired
    private ExamQuestionDao examQuestionDao;

    @Override
    public ReportSubjectQuestion save(ReportSubjectQuestion reportSubjectQuestion) {
        return reportSubjectQuestionDao.save(reportSubjectQuestion);
    }

    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectQuestionDao.deleteByExamId(examId);

    }

    @Override
    public List<ReportSubjectQuestion> findByQuery(final ReportSubjectQuery query) {
        List<ReportSubjectQuestion> result = reportSubjectQuestionDao.findAll(new Specification<ReportSubjectQuestion>() {
            @Override
            public Predicate toPredicate(Root<ReportSubjectQuestion> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (query.getSubjectCode() != null) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                predicates.add(cb.equal(root.get("objective"), query.getObjective()));
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        });
        return result;
    }

    @Override
    public void saveReportSubjectObjectiveQuestionData(Integer examId, String subjectCode, Boolean objective) {

        List<ExamQuestion> examQuestionObjectiveList = examQuestionDao.findByExamIdAndSubjectCodeAndObjective(examId, subjectCode, objective);

        List<ExamStudent> examStudentList = examStudentDao.findByExamIdAndSubjectCodeAndUploadTimeNotNullAndAbsent(examId, subjectCode, false);

        Integer studentCount = examStudentList.size();

        for (int i = 0; i < examQuestionObjectiveList.size(); i++) {
            ExamQuestion question = examQuestionObjectiveList.get(i);
            if (question.getTotalScore().doubleValue() > 0) {
                ReportSubjectQuestion reportSubjectQuestion = new ReportSubjectQuestion();
                //得分人数
                Integer scoreCount = 0;
                //满分人数
                Integer fullScoreCount = 0;
                //考生题目得分汇总
                Double sumScore = 0d;
                //题目总分汇总
                Double questionSumScore = DoubleUtil.mul(question.getTotalScore(), studentCount);
                //方差
                Double variance = 0d;

                for (int j = 0; j < studentCount; j++) {
                    ExamStudent student = examStudentList.get(j);
                    Double score = 0d;
                    if (objective) {
                        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
                            //客观题
                            String[] objectiveAnswers = student.getObjectiveScoreList().split(";");
                            score = Double.parseDouble(objectiveAnswers[i].split(":")[1]);
                        }
                    } else {
                        if (StringUtils.isNotBlank(student.getSubjectiveScoreList())) {
                            //主观题
                            String[] subjectiveAnswers = student.getSubjectiveScoreList().split(";");
                            score = Double.parseDouble(subjectiveAnswers[i]);
                        }
                    }
                    sumScore = DoubleUtil.sum(sumScore, score);
                    if (score > 0) {
                        scoreCount++;
                    }
                    if (score.doubleValue() == question.getTotalScore().doubleValue()) {
                        fullScoreCount++;
                    }
                }
                Double avgScore = DoubleUtil.div(sumScore, examStudentList.size(), 2);
                for (int j = 0; j < studentCount; j++) {
                    ExamStudent student = examStudentList.get(j);
                    Double score = 0d;
                    if (objective) {
                        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
                            //客观题
                            String[] objectiveAnswers = student.getObjectiveScoreList().split(";");
                            score = Double.parseDouble(objectiveAnswers[i].split(":")[1]);
                        }
                    } else {
                        if (StringUtils.isNotBlank(student.getSubjectiveScoreList())) {
                            //主观题
                            String[] subjectiveAnswers = student.getSubjectiveScoreList().split(";");
                            score = Double.parseDouble(subjectiveAnswers[i]);
                        }
                    }
                    variance = DoubleUtil.sum(variance, DoubleUtil.mul(DoubleUtil.sub(score, avgScore), DoubleUtil.sub(score, avgScore)));
                }
                reportSubjectQuestion.setExamId(examId);
                reportSubjectQuestion.setSubjectCode(subjectCode);
                reportSubjectQuestion.setObjective(objective);
                reportSubjectQuestion.setQuestionName(question.getMainTitle() + "_" + (i + 1));
                reportSubjectQuestion.setTotalScore(question.getTotalScore());
                reportSubjectQuestion.setAvgScore(avgScore);
                reportSubjectQuestion.setScoreRate(DoubleUtil.div(scoreCount, studentCount, 2));
                reportSubjectQuestion.setSd(DoubleUtil.format(Math.sqrt(variance / studentCount), "0.00"));
                reportSubjectQuestion.setFullScoreRate(DoubleUtil.div(fullScoreCount, studentCount, 2));
                reportSubjectQuestionDao.save(reportSubjectQuestion);
            }
        }

    }


}
