package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectRangeDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectRange;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectRangeService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.LinkedList;
import java.util.List;

/**
 * 课程分段统计 service 实现
 *
 * @author xuwenjin
 * @Date 2019-04-19
 */
@Service
public class ReportSubjectRangeServiceImpl extends BaseQueryService<ReportSubjectRange> implements ReportSubjectRangeService {

    @Autowired
    private ReportSubjectRangeDao reportSubjectRangetDao;


    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectRangetDao.deleteByExamId(examId);
    }

    @Override
    public ReportSubjectQuery findByQuery(final ReportSubjectQuery query) {
        checkQuery(query);
        Page<ReportSubjectRange> result = reportSubjectRangetDao.findAll(new Specification<ReportSubjectRange>() {

            @Override
            public Predicate toPredicate(Root<ReportSubjectRange> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();

                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        }, query);
        fillResult(result, query);

        return query;
    }

    /**
     * 保存 课程分段统计
     *
     * @param examId 考试Id
     */
    @Override
    public void saveReportRangeSubjectData(Integer examId) {
        List<Object[]> datas = reportSubjectRangetDao.queryRangeSubjectByExamId(examId);
        for (Object[] obj : datas) {
            ReportSubjectRange subjectRange = new ReportSubjectRange();
            subjectRange.setExamId(examId);
            subjectRange.setSubjectCode(obj[0].toString());
            subjectRange.setSubjectName(obj[1].toString());
            subjectRange.setLevel(obj[2] != null ? obj[2].toString() : "");
            subjectRange.setCount0_49(Integer.parseInt(obj[3].toString()));
            subjectRange.setCount50_59(Integer.parseInt(obj[4].toString()));
            subjectRange.setCount60_69(Integer.parseInt(obj[5].toString()));
            subjectRange.setCount70_79(Integer.parseInt(obj[6].toString()));
            subjectRange.setCount80_89(Integer.parseInt(obj[7].toString()));
            subjectRange.setCount90_100(Integer.parseInt(obj[8].toString()));
            subjectRange.setLt60(Integer.parseInt(obj[9].toString()));
            subjectRange.setMte60(Integer.parseInt(obj[10].toString()));
            subjectRange.setTotalCount(Integer.parseInt(obj[11].toString()));
            subjectRange.setPercent0_49(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount0_49(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent50_59(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount50_59(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent60_69(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount60_69(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent70_79(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount70_79(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent80_89(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount80_89(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent90_100(DoubleUtil.format(DoubleUtil.div(subjectRange.getCount90_100(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent_lt60(DoubleUtil.format(DoubleUtil.div(subjectRange.getLt60(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            subjectRange.setPercent_mte60(DoubleUtil.format(DoubleUtil.div(subjectRange.getMte60(), subjectRange.getTotalCount(), 4) * 100, "0.00"));
            reportSubjectRangetDao.save(subjectRange);
        }
    }


}
